from setuptools import setup, find_packages

setup(
    name='multimodal_sdk',
    version='0.0.5',
    packages=find_packages(),
    install_requires=[
        # Add any SDK-specific dependencies here
    ],
)
